<?php
define( 'WP_USE_THEMES', false );
require_once( dirname( __FILE__ ) . '/../../../../wp-blog-header.php' );
require_once( 'open_content.php' );
require_once( 'oc_api.php' );

$inputJSON = file_get_contents('php://input');

if( validate_remote_token() && $inputJSON !== '' ) {
    new OcPush( $inputJSON );
}

function validate_remote_token() {
    $token = md5( strtolower( plugin_dir_url(__FILE__) . "oc_push.php" ) );
    $remote_token = $_GET['token'];

    return $token === $remote_token;
}

class OcPush {
    const OC_ADD = 'ADD';
    const OC_UPDATE = 'UPDATE';
    const OC_DELETE = 'DELETE';
	const OC_PUSH_DEBUG = 'OC_PUSH_DEBUG';

    private $oc = null;
    private $raw_response;
    private $decoded_response;
    private $contenttype;
    private $raw_contenttype;
    private $eventtype;
    private $uuid;

    function __construct( $input ) {

        $this->raw_response = $input;
        $this->oc = new OcAPI();
        $this->decoded_response = json_decode( $input );
        $this->contenttype = strtolower( $this->decoded_response->contentType );
        $this->raw_contenttype = $this->decoded_response->contentType;
        $this->eventtype = $this->decoded_response->eventtype;
        $this->uuid = $this->decoded_response->uuid;

        // Fetch data for object from OC to get hierarchical properties as well.
        $object_data = $this->get_object_data();
        if( isset( $object_data ) && is_string( $object_data ) ) {
            $this->raw_response = $object_data;
        }

        switch( $this->eventtype ) {
            case self::OC_ADD:
                $this->on_oc_add();
                break;
            case self::OC_UPDATE:
                $this->on_oc_update();
                break;
            case self::OC_DELETE:
                $this->on_oc_delete();
                break;
        }

        $this->add_to_debug();
        OcCacheHelper::getInstance()->clean_up_cache_map( $this->uuid );
        OcCacheHelper::getInstance()->clear_widget_cache_by_uuid( $this->uuid );
    }

	private function add_to_debug() {
		$time = strtotime( 'now' );
		$current_debug = json_decode( get_transient( self::OC_PUSH_DEBUG ), true );

		if( $current_debug !== false && !empty( $current_debug ) ) {
			if( sizeof( $current_debug ) > 4 ) {
				array_pop( $current_debug );
			}

			array_unshift( $current_debug, array( 'time' => $time, 'content' => $this->contenttype, 'uuid' => $this->uuid, 'event' => $this->eventtype ) );
			set_transient( self::OC_PUSH_DEBUG, json_encode( $current_debug ) );
		} else {
			set_transient( self::OC_PUSH_DEBUG, json_encode( array ( array( 'time' => $time, 'content' => $this->contenttype, 'uuid' => $this->uuid, 'event' => $this->eventtype ) ) ) );
		}

	}

    private function on_oc_update() {

        //update cache
        $this->oc->update_transient_notifier_push( $this->get_push_response() );

        do_action( 'oc_push_update', $this->get_push_response() );
    }

    private function on_oc_add() {

        //add single article cache
        if( $this->contenttype === 'article' )
            $this->oc->add_transient_notifier_push( $this->get_push_response() );

        do_action( 'oc_push_add', $this->get_push_response() );
    }

	private function on_oc_delete() {
		//delete from cache
		$this->oc->delete_transient_notifier_push( $this->uuid );

		//delete post
		$post = $this->get_wp_post( $this->uuid );
		if ( !empty( $post ) && isset( $post[0], $post[0]->ID ) && $post[0] !== null ) {
			$post_uuid = get_post_meta( $post[0]->ID, 'oc_uuid', true );
			if( $post_uuid == $this->uuid ) {
				wp_delete_post( $post[0]->ID );
			}
		}

		delete_transient( 'pe_' . $this->uuid );
		delete_transient( 'the_post_id_' . $this->uuid );
		delete_transient( 'the_post_' . $this->uuid );

		do_action( 'oc_push_delete', $this->get_push_response() );
	}

	private function get_object_data() {

        $properties = $this->oc->get_contenttype_properties( $this->raw_contenttype, [ 'boolean', 'date', 'double', 'integer', 'long', 'stream', 'string' ] );
        $filtered_properties = apply_filters( 'ew_notifier_update_' . $this->contenttype . '_properties', $properties );

        // If no difference in filtered properties, no need to fetch data from Open Content.
        if($properties === $filtered_properties) {
            return null;
        }

        $object_data = $this->oc->get_single_object_data( $this->uuid, $filtered_properties, false );

        return $object_data;
    }

    private function get_push_response() {
        return array( 'eventtype' => $this->eventtype, 'uuid' => $this->uuid, 'response' => $this->raw_response );
    }

    private function get_wp_post( $uuid ) {
	    $article_arr    = [];
        $article_arr[0] = OcUtilities::get_article_post_by_uuid( $uuid );
        return $article_arr;
    }
}